@@ -1206,6 +1206,31 @@ rcptto(SMFICTX *ctx, char **to)
 	return SMFIS_TEMPFAIL;
     }
 
+    /* Apparently, Postfix offers an option to set the "i" macro at
+       rcptto time */
+    if (!data->qid || (data->qid == NOQUEUE)) {
+	char *queueid = smfi_getsymval(ctx, "i");
+	if (queueid && *queueid) {
+	    if (!data->qid_written) {
+		/* Write this out separately; the write below may be skipped */
+		data->cmdFD = get_fd(data, "COMMANDS", data->cmdFD);
+		if (data->cmdFD >= 0) {
+		    dbuf_init(&dbuf);
+		    append_mx_command(&dbuf, 'Q', queueid);
+		    if (write_dbuf(&dbuf, data->cmdFD, data, "COMMANDS") >= 0) {
+			data->qid_written = 1;
+		    }
+		    dbuf_free(&dbuf);
+		    data->cmdFD = put_fd(data->cmdFD);
+		}
+	    }
+	    data->qid = strdup_with_log(queueid);
+	    if (!data->qid) {
+		data->qid = NOQUEUE;
+	    }
+	}
+    }
+
     rcpt_mailer = smfi_getsymval(ctx, "{rcpt_mailer}");
     if (!rcpt_mailer || !*rcpt_mailer) rcpt_mailer = "?";
 
